#!/usr/bin/perl -w
# XChat plugin for on-screen display using xosd
# Copyright (C) 2002  Dave O'Neill <dmo@acm.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.

## You will need to download the xosd library from http://www.ignavus.net/software.html
## as well as the X::Osd package from http://www.cpan.org/authors/id/G/GO/GOZER/X-Osd-0.02.tar.gz

package IRC::Xchat::osd;
use X::Osd;

my $script_name = "osd.pl";
my $script_version = "0.5-Custom-R1";

my $channel_info = {};

my $savefile = $ENV{'HOME'}."/.xchat/osd.conf";
my $is_on = 0;

my $colour = 'green';
my $delay  = 3;   ## delay in seconds.
my $voffset = 37;
my $hoffset = 1;
my $soffset = 2;
my $osd;

my $handlers = { 
	'on'	=> \&on,
	'off'	=> \&off,
	'delay'		=> \&delay,
	'colour'	=> \&colour,
	'color'		=> \&colour, ## Damn Americans.
	'location'  => \&location,
	'loc'  => \&location,
	'voffset' => \&voffset,
	'hoffset' => \&hoffset,
	'shadow' => \&shadow,
	'test' => \&test,
	'help'	=> \&help,
	'' => \&help
};

IRC::print "Loaded XChat OSD plugin\ \n";
IRC::print "/osd help for more information \ \n";
IRC::register ("OSD Script", ".4", "IRC::Xchat::osd::cleanup", "");
IRC::add_command_handler("osd", "IRC::Xchat::osd::cmd_handler");
IRC::add_message_handler("PRIVMSG", "IRC::Xchat::osd::display_incoming_text");
init();

sub init
{
	$osd = X::Osd->new("-urw-avantgarde-demibold-r-normal-*-*-260-*-*-p-*-iso8859-1",$colour,$delay,XOSD_bottom,1,2);
	voffset($voffset);
	hoffset($hoffset);
	shadow($soffset);
	on();
	osd("OSD plugin version " . $script_version . " is active\n");
	delay(5);
}

sub cmd_handler 
{
	my $args = shift;
	$args =~ s/\s+/ /g;
	my ($cmd,@args) = split(/\s/,$args);
	if(exists($handlers->{$cmd})) {
		$handlers->{$cmd}->(@args);
	} else {
		IRC::print "! $cmd is not a valid command \n";
	}

	return 1;
}

sub help
{
 	IRC::print "##################################################\n";
 	IRC::print "########## Support for xosd in XChat #############\n";
 	IRC::print "##################################################\n";
	IRC::print "Available commands are: \n";
	IRC::print "/osd help\ \n";
	IRC::print "    - this command\ \n";
	IRC::print "/osd on\ \n";
	IRC::print "    - turns on osd support\ \n";
	IRC::print "/osd off\ \n";
	IRC::print "    - turns off osd support\ \n";
	IRC::print "/osd colour <colour>\ \n";
	IRC::print "    - Changes colour of text.  <colour> should be a named colour known to X\ \n";
	IRC::print "/osd delay <seconds>\ \n";
	IRC::print "    - Changes on-screen time of message.  <seconds> should be an integer number of seconds \n";
	IRC::print "/osd location (top|bottom)\ \n";
	IRC::print "    - Changes location of text on screen (default is bottom)\ \n";
    IRC::print "/osd voffset <offset>\ \n";
	IRC::print "    - Changes the vertical offset of text on screen\ \n";
    IRC::print "/osd hoffset <offset>\ \n";
	IRC::print "    - Changes the horizontal offset of text on screen\ \n";
	IRC::print "/osd shadow <offset>\ \n";
    IRC::print "    - Changes the shadow offset of text on screen\ \n";
    IRC::print "/osd test\ \n";
	IRC::print "    - Write a sample test message \ \n";
#	IRC::print "/osd load [filename]\ \n";
#	IRC::print "    - load config from filename (default is $savefile)\ \n";
#	IRC::print "/osd save [filename]\ \n";
#	IRC::print "    - save current config to filename (default is $savefile)\ \n";
}


sub test
{
	osd("OSD: Sample message\n");
}

sub on 
{
	if(! $is_on ) {
		$is_on = 1;
		IRC::print "OSD activated \n";
	} else {
		IRC::print "OSD already active \n";
	}
}

sub off
{
	if($is_on ) {
		$is_on = 0;
		IRC::print "OSD deactivated \n";
	} else {
		IRC::print "OSD already deactivated \n";
	}
}

sub colour
{
	my $c = shift || do {
		IRC::print "You must supply a colour name \n";
		return;
	};
	IRC::print "Setting colour to $c \n";
	$colour = $c;
	$osd->set_colour($c);
}

sub voffset
{
        my $d = shift || do {
                IRC::print "You must supply a vertical offset \n";
                return;
        };
        IRC::print "Setting vertical offset to $d \n";
        $voffset =  $d;
        $osd->set_vertical_offset($d);
	osd("Testing the new vertical offset\n");
}

sub hoffset
{
        my $d = shift || do {
                IRC::print "You must supply a horizontal offset \n";
                return;
        };
        IRC::print "Setting horizontal offset to $d \n";
        $hoffset =  $d;
        $osd->set_horizontal_offset($d);
        osd("Testing the new horizontal offset\n");
}

sub shadow
{
        my $d = shift || do {
                IRC::print "You must supply a shadow offset \n";
                return;
        };
        IRC::print "Setting shadow offset to $d \n";
        $soffset =  $d;
        $osd->set_shadow_offset($d);
        osd("Testing the shadow offset\n");
}
sub delay
{
	my $d = shift || do {
		IRC::print "You must supply a delay \n";
		return;
	};
	IRC::print "Setting delay timeout to $d seconds \n";
	$delay = $d;
	$osd->set_timeout($d);
}

sub location
{
	my $loc = shift;
	
	if($loc =~ m/top/i) {
		$osd->set_pos(XOSD_top);
		IRC::print "OSD will now appear at the top of your screen \n";
	} elsif($loc =~ m/bot(tom)?/i) {
		$osd->set_pos(XOSD_bottom);
		IRC::print "OSD will now appear at the bottom of your screen \n";
	} else {
		IRC::print "You must specify top or bottom \n";
	}
}

sub display_incoming_text
{

	my $line = shift(@_);
	if ($is_on)
	{
		my $me = IRC::get_info(1);

		my ($speaker,$addr,$type,$target,$txt) = $line =~ /:(.*?)!(.*?@.*?)\s([^\s]+)\s([^\s]+)\s:(.*)/;
		if($line =~ /\b$me\b/) {
			my $msg = "";
			if($target =~ /^#/) {
				## Said in a channel, so prepend channel name
				$msg .= "$target ";
			}
			if($txt =~ s/ACTION/$speaker/g) {
				
				$msg .= $txt;
			} else {
				## Remove leading name, if present
				$txt =~ s/^\s*$me\s*[:,]?//;
				$msg .= "<$speaker>: $txt";
			}
			osd($msg);
		}
	}

	return 0;
}

sub osd
{
	my $msg = shift;
	$osd->string(0,$msg);
}

sub cleanup
{
	IRC::print "Unloaded xosd plugin \n";
}
